package com.blogspot.pontoflutuante;

import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;

/**
  * Class to handle and process http file uploads.
  *
  *
  * @author Daniel Cristiano Rocha <a href="dcrocha@uai.com.br">dcrocha@uai.com.br</a>
  * @version 0.2
  *
  */

public abstract class FileUpload {
	/*
	 * All these variables were private in Daniel's code, i have made them protected
	 * - raditha
	 */
	protected String err = "";
	protected HttpServletRequest request;
	protected HashMap collection;
	protected String saveDir;

	public FileUpload(HttpServletRequest request) {
		this.request = request;
		this.collection = new HashMap();
		}

	/**

		Sets the directory path where files should be saved, for all MimeBodyPart objects
		which are files, within the HashMap.Creates directory specified by saveDir if it does not exist.

		Has no effect if upload method has not been called yet.

		Can be overriden on an individual basis by MimeBodyPart.setSaveDir(String saveDir)
		method. Added in version 0.2

		@param saveDir : String representing a directory path. (you should escape \ characters)
		@return void
	  */



	public void setSaveDir(String saveDir) {

		Collection ks = collection.values();
		Iterator i = ks.iterator();

		this.saveDir = saveDir;

		File f = new File(saveDir);
		System.out.println("saving files to " + saveDir);
		if(!f.exists()) {
			    f.mkdirs(); //creates directory specified by saveDir if it does not exist
			}

		while(i.hasNext()) {
			Object o = i.next();
			if(o instanceof MimeBodyPart) {
				MimeBodyPart mbp = (MimeBodyPart)o;
					if(mbp != null && mbp.isFile()) {
						mbp.setSaveDir(saveDir);
					}
				} // instanceof if
			else {
				Vector v = (Vector)o;
				for(int j=0;j<v.size();j++) {
					MimeBodyPart mbp = (MimeBodyPart)v.get(j);
						if(mbp != null && mbp.isFile()) {
							mbp.setSaveDir(saveDir);
						}
					}
				}
			}

		}

	/**
	  Returns this object's current path for saving files. Added in version 0.2

	  @return A String representing directory path.
	  */

	public String getSaveDir() {
		return saveDir;
		}

	/**
		Returns a String that shows the last error message. Should be used when upload()
		returns false.

		@return A String representing the last error message.

	  */

	public String getErrString() {
		return err;
		}
	/**
		Parse all http content and populates the HashMap <i>collection</i> field with correspondent
		MimeBodyPart objects.

		@return A boolean true if the upload was successful, and false otherwise.

	  */


	public boolean upload() {
		getBoundary();
		return (err.equals(""));
		}

	/**
		Maybe the most important method beyond upload. getMimeParts() returns a HashMap
		object that can contain two types of objects :

			- MimeBodyPart objects : this means that there's only ONE field with a certain
			  name on the HTML form.
			- Vector objects : this means that there is more than one form field with a certain
			  name (like a &lt;select multiple>&lt;/select>, for example) within the form, then this Vector
			  holds all MimeBodyPart objects found under that name.

		To find out whether there are single or multiple form fields with the same name, you
		should use the following code :

			<pre>
			HashMap m = fileuploadinstance.getMimeParts();
			Object c = m.get("FieldName");
			if(c instanceof Vector) {
				// loop thru the Vector filled with MimeBodyParts
				}
			else {
				MimeBodyPart part = (MimeBodyPart)c;
				out.println(part.getName()) // etc.. etc. etc.
				}
			</pre>

		@ return A HashMap object containing all of the MimeBodyParts found.

	   */

	public HashMap getMimeParts() {
		return (HashMap)collection.clone();
		}

	/* modo private para extrair o boundary . chama parseBody depois. */

	private void getBoundary()  {

		String contentType = request.getContentType();
		String boundary = "";
		final int BOUNDARY_WORD_SIZE = "boundary=".length();

		if(contentType == null || !contentType.startsWith("multipart/form-data")) {
			err = "Ilegal ENCTYPE : must be multipart/form-data\n";
			err += "ENCTYPE set = " + contentType;
			}
		else {
			boundary = contentType.substring(contentType.indexOf("boundary=") + BOUNDARY_WORD_SIZE);
			boundary = "--" + boundary; // do protocolo
			parseBody(boundary);
			}

		}

	/* modo private que pega todo o contedo postado, separa pelo boundary,
	   e criando vios objetos MimeBodyPart que ser colocados no vector mimeParts */

	public abstract void parseBody(String boundary);

	/* modo para retirar 2 bytes do ByteArrayOutputStream = "\r\n" */
	/* this method was privet in Daniel's code -raditha*/
	protected byte[] setByteArray(ByteArrayOutputStream baos) {
		byte[] temp;
		byte[] ret = null;
		if(baos != null) {
			temp = baos.toByteArray();
			ret = new byte[temp.length - 2];
			System.arraycopy(temp,0,ret,0,temp.length - 2);
			temp = null;
			}
		return ret;
		}

	/* modo para ajudar a fazer o setup do hashmap com os nomes.
	   basicamente, o que ele faz :
	   		- se o nome do mimeBodyPart nico, ele o inserta no hashmap direto
			- se n for nico, cria um vetor de mimeBodyParts sob um determinado
			nome (o que se repete no form) e o encaixa dentro dele.

	   Sendo assim, o retorno do hashmap pode ser um objeto MimeBodyPart ou
	   um Vector.

	 */

	/* this method was private in Daniel's code */
	protected void setupHashMap(MimeBodyPart m) {
		if(m != null) {
			String nome = m.getName();
			Object c;
			if((c=collection.get(nome)) == null) {  // nome de form nico
				collection.put(nome,m);
				} // if de collection
			else {
				if(c instanceof Vector) { //jtem um vetor especificado
					Vector temp = (Vector)c;
					temp.add(m);
					collection.put(nome,temp); // pe o vetor na collection
					}
				else {
					Vector temp = new Vector(); //cria novo vetor
					temp.add(c); // adiciona objeto antigo no vetor
					temp.add(m); // adiciona m no vetor
					collection.put(nome,temp); //adiciona vetor no hashMap (substitui a entrada anterior)
					}
				}
			} // m==null
		}


	} // fileupload
